package com.fjhx.mail.service.personal.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.mail.entity.personal.dto.PersonalMailboxDto;
import com.fjhx.mail.entity.personal.dto.PersonalMailboxSelectDto;
import com.fjhx.mail.entity.personal.po.PersonalFolder;
import com.fjhx.mail.entity.personal.po.PersonalMailbox;
import com.fjhx.mail.entity.personal.vo.PersonalMailboxVo;
import com.fjhx.mail.mapper.personal.PersonalMailboxMapper;
import com.fjhx.mail.service.personal.PersonalFolderService;
import com.fjhx.mail.service.personal.PersonalMailboxService;
import com.fjhx.mail.util.EmailUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.sun.mail.imap.IMAPStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.mail.MessagingException;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 个人邮箱 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-06
 */
@Service
public class PersonalMailboxServiceImpl extends ServiceImpl<PersonalMailboxMapper, PersonalMailbox> implements PersonalMailboxService {

    @Autowired
    private PersonalFolderService personalFolderService;

    @Override
    public Page<PersonalMailboxVo> getPage(PersonalMailboxSelectDto dto) {
        IWrapper<PersonalMailbox> wrapper = getWrapper();
        wrapper.eq("pm", PersonalMailbox::getUserId, SecurityUtils.getUserId());
        wrapper.keyword(dto, new SqlField("pm", PersonalMailbox::getMailUser));
        wrapper.orderByDesc("pm", PersonalMailbox::getId);
        Page<PersonalMailboxVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public PersonalMailboxVo detail(Long id) {
        PersonalMailbox PersonalMailbox = this.getById(id);
        PersonalMailboxVo result = BeanUtil.toBean(PersonalMailbox, PersonalMailboxVo.class);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PersonalMailboxDto personalMailboxDto) {

        Long personalMailboxId = IdWorker.getId();

        try {
            IMAPStore imapStore = EmailUtil.getIMAPStore(
                    personalMailboxDto.getReceiveHost(),
                    personalMailboxDto.getReceivePort(),
                    personalMailboxDto.getMailUser(),
                    personalMailboxDto.getMailPassword()
            );

            List<String> folderNameList = EmailUtil.getFolderNameList(imapStore);
            List<PersonalFolder> personalFolderList = folderNameList.stream().map(item -> {
                PersonalFolder personalFolder = new PersonalFolder();
                personalFolder.setMailboxId(personalMailboxId);
                personalFolder.setName(item);
                personalFolder.setSyncStatus(1);
                return personalFolder;
            }).collect(Collectors.toList());

            personalFolderService.saveBatch(personalFolderList);

        } catch (MessagingException e) {
            throw new ServiceException("连接邮箱异常");
        }

        personalMailboxDto.setId(personalMailboxId);
        personalMailboxDto.setStatus(StatusConstant.ENABLE);
        personalMailboxDto.setSyncStatus(1);
        personalMailboxDto.setDefaultStatus(0);
        personalMailboxDto.setUserId(SecurityUtils.getUserId());

        this.save(personalMailboxDto);
    }

    @Override
    public void edit(PersonalMailboxDto personalMailboxDto) {
        this.updateById(personalMailboxDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
