package com.fjhx.sale.controller.contract;

import com.fjhx.sale.entity.contract.po.Contract;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.contract.dto.ContractSelectDto;
import com.fjhx.sale.entity.contract.dto.ContractDto;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.fjhx.sale.service.contract.ContractService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * <p>
 * 外销合同表 前端控制器
 * </p>
 *
 * @author 
 * @since 2023-04-17
 */
@RestController
@RequestMapping("/contract")
public class ContractController {

    @Autowired
    private ContractService contractService;

    /**
     * 外销合同表分页
     */
    @PostMapping("/page")
    public Page<ContractVo> page(@RequestBody ContractSelectDto dto) {
        return contractService.getPage(dto);
    }

    /**
     * 外销合同表明细
     */
    @PostMapping("/detail")
    public ContractVo detail(@RequestBody BaseSelectDto dto) {
        return contractService.detail(dto.getId());
    }

    /**
     * 外销合同表新增
     */
    @PostMapping("/add")
    public void add(@RequestBody ContractDto contractDto) {
        contractService.add(contractDto);
    }

    /**
     * 外销合同表编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody ContractDto contractDto) {
        contractService.edit(contractDto);
    }

    /**
     * 外销合同表删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        contractService.delete(dto.getId());
    }

    /**
     * 根据客户ID查询未包装的订单
     */
    @GetMapping("/getNoPackContractByCustomerId")
    public List<ContractVo> getNoPackContractByCustomerId(@RequestParam("customerId")String customerId) {
        return contractService.getNoPackContractByCustomerId(customerId);
    }

    /**
     * 根据合同ID和客户ID查询未包装的产品
     */
    @GetMapping("/getNoPackContractProductById")
    public List<ContractVo> getNoPackContractProductById(@RequestParam("customerId")String customerId,@RequestParam("contractId")String contractId) {
        return contractService.getNoPackContractProductById(customerId,contractId);
    }

    /**
     * 根据合同id集合查询合同列表
     */
    @PostMapping("/getByIds")
    public List<Contract> getByIds(@RequestBody List<Long> ids) {
        return contractService.getByIds(ids);
    }


}
