package com.fjhx.sale.flow;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.purchase.entity.purchase.enums.PurchaseDataResourceEnum;
import com.fjhx.purchase.entity.purchase.enums.PurchaseDetailStatusEnum;
import com.fjhx.purchase.entity.purchase.enums.PurchaseStatusEnum;
import com.fjhx.purchase.entity.purchase.po.Purchase;
import com.fjhx.purchase.entity.purchase.po.PurchaseDetail;
import com.fjhx.purchase.service.purchase.PurchaseDetailService;
import com.fjhx.purchase.service.purchase.PurchaseService;
import com.fjhx.purchase.util.code.CodeEnum;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.service.contract.ContractProductService;
import com.obs.services.internal.ServiceException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 采购流程
 *
 * @Author:caozj
 * @DATE:2023/4/3 17:38
 */
@Component
public class PurchaseFlow extends FlowDelegate {

    @Autowired
    private PurchaseService purchaseService;

    @Autowired
    private PurchaseDetailService purchaseDetailService;

    @Autowired
    private ContractProductService contractProductService;

    @Override
    public String getFlowKey() {
        return "purchase_flow";
    }

    /**
     * 发起流程
     * @param flowId 流程ID
     * @param submitData 采购数据
     * @return
     */
    @Override
    public Long start(Long flowId, JSONObject submitData) {
        DynamicDataSourceContextHolder.push(SourceConstant.PURCHASE);
        Purchase purchase = submitData.toJavaObject(Purchase.class);
        purchase.setCode(CodeEnum.PURCHASE.getCode());
        purchase.setPurchaseStatus(PurchaseStatusEnum.UNDER_REVIEW.getKey());
        purchaseService.save(purchase);
        List<PurchaseDetail> purchaseDetailList = purchase.getPurchaseDetailList();
        if(CollectionUtils.isNotEmpty(purchaseDetailList)){
            for(PurchaseDetail s : purchaseDetailList){
                s.setPurchaseId(purchase.getId());
            }
            purchaseDetailService.saveBatch(purchaseDetailList);
        }
        DynamicDataSourceContextHolder.poll();
        return purchase.getId();
    }

    /**
     * 结束流程
     * @param flowId 流程ID
     * @param businessId 业务ID
     * @param submitData 数据
     */
    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        //通过业务ID查询采购数据
        Purchase purchase = purchaseService.getById(businessId);
        if(ObjectUtils.isEmpty(purchase)){
            throw new ServiceException("采购单不存在");
        }
        //查询采购产品
        List<PurchaseDetail> purchaseDetailList = purchaseDetailService.list(Wrappers.<PurchaseDetail>query().lambda().eq(PurchaseDetail::getPurchaseId,businessId));
        List<ContractProduct> upContractProduct = new ArrayList<>();
        for(PurchaseDetail p:purchaseDetailList){
            if(ObjectUtils.isNotEmpty(p.getDataResourceId())&&
                    p.getDataResource()== PurchaseDataResourceEnum.DATA_RESOURCE_1.getKey()){//如果采购的是外销合同
                ContractProduct contractProduct = contractProductService.getById(p.getDataResourceId());
                BigDecimal expendQuantity = contractProduct.getExpendQuantity().subtract(p.getCount());
                if(expendQuantity.compareTo(BigDecimal.ZERO)< 1){//小于0不让继续执行
                    throw new ServiceException("采购数量不得大于外销合同数量");
                }
                contractProduct.setExpendQuantity(expendQuantity);
                upContractProduct.add(contractProduct);
            }
        }
        if(CollectionUtils.isNotEmpty(upContractProduct)){//扣减销售合同数量
            contractProductService.updateBatchById(upContractProduct);
        }
        //修改采购状态为审批通过
        purchase.setPurchaseStatus(PurchaseStatusEnum.PASS.getKey());
        purchase.setApprovedDate(new Date());
        purchaseService.updateById(purchase);
        //修改采购明细为待采购
        PurchaseDetail detail = new PurchaseDetail();
        detail.setStatus(PurchaseDetailStatusEnum.PASS.getKey());
        purchaseDetailService.update(detail,Wrappers.<PurchaseDetail>query()
                .lambda().eq(PurchaseDetail::getPurchaseId,purchase.getId()));
    }

}
