package com.fjhx.sale.service.serviceContract.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.sale.entity.serviceContract.dto.ServiceContractDto;
import com.fjhx.sale.entity.serviceContract.dto.ServiceContractSelectDto;
import com.fjhx.sale.entity.serviceContract.po.ServiceContract;
import com.fjhx.sale.entity.serviceContract.po.ServiceContractPay;
import com.fjhx.sale.entity.serviceContract.po.ServiceContractProduct;
import com.fjhx.sale.entity.serviceContract.vo.ServiceContractVo;
import com.fjhx.sale.mapper.serviceContract.ServiceContractMapper;
import com.fjhx.sale.service.serviceContract.ServiceContractPayService;
import com.fjhx.sale.service.serviceContract.ServiceContractProductService;
import com.fjhx.sale.service.serviceContract.ServiceContractService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 服务合同表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-18
 */
@Service
public class ServiceContractServiceImpl extends ServiceImpl<ServiceContractMapper, ServiceContract> implements ServiceContractService {

    @Autowired
    private ServiceContractProductService serviceContractProductService;

    @Autowired
    private ServiceContractPayService serviceContractPayService;

    /**
     * 服务合同表分页
     */
    @Override
    public Page<ServiceContractVo> getPage(ServiceContractSelectDto dto) {
        QueryWrapper<Object> query = Wrappers.query();
        query.orderByDesc("sc.create_time");
        query.eq(ObjectUtil.isNotEmpty(dto.getStatus()),"sc.status",dto.getStatus());
        query.eq(ObjectUtil.isNotEmpty(dto.getServiceStatus()),"cc.status",dto.getServiceStatus());
        Page<ServiceContractVo> page = this.baseMapper.getPage(dto.getPage(), query);
        List<ServiceContractVo> records = page.getRecords();
        //赋值用户名称
        UserUtil.assignmentNickName(records, BasePo::getCreateUser, ServiceContractVo::setUserName);
        return page;
    }

    /**
     * 服务合同表明细
     */
    @Override
    public ServiceContractVo detail(Long id) {
        //查询服务合同表的信息
        ServiceContract ServiceContract = this.getById(id);
        ServiceContractVo result = BeanUtil.toBean(ServiceContract, ServiceContractVo.class);

        //查询服务合同-商品表的数据
        List<ServiceContractProduct> contractProductList = serviceContractProductService.list(Wrappers.<ServiceContractProduct>lambdaQuery()
                .eq(ServiceContractProduct::getServiceContractId, result.getId()));

        //查询服务合同-收费项目
        List<ServiceContractPay> contractPayList = serviceContractPayService.list(Wrappers.<ServiceContractPay>lambdaQuery()
                .eq(ServiceContractPay::getServiceContractId, result.getId()));

        result.setServiceContractProductList(contractProductList);
        result.setServiceContractPayList(contractPayList);
        return result;
    }

    @Override
    public void add(ServiceContractDto serviceContractDto) {
        this.save(serviceContractDto);
    }

    @Override
    public void edit(ServiceContractDto serviceContractDto) {
        this.updateById(serviceContractDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
