package com.fjhx.victoriatourist.service.deliver.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.purchase.entity.purchase.po.Purchase;
import com.fjhx.purchase.entity.purchase.po.PurchaseDetail;
import com.fjhx.purchase.service.purchase.PurchaseDetailService;
import com.fjhx.purchase.service.purchase.PurchaseService;
import com.fjhx.victoriatourist.entity.deliver.dto.DeliverGoodsDto;
import com.fjhx.victoriatourist.entity.deliver.dto.DeliverGoodsSelectDto;
import com.fjhx.victoriatourist.entity.deliver.po.DeliverGoods;
import com.fjhx.victoriatourist.entity.deliver.po.DeliverGoodsDetails;
import com.fjhx.victoriatourist.entity.deliver.vo.DeliverGoodsDetailsVo;
import com.fjhx.victoriatourist.entity.deliver.vo.DeliverGoodsVo;
import com.fjhx.victoriatourist.entity.logistics.po.LogisticsDetails;
import com.fjhx.victoriatourist.entity.logistics.po.LogisticsInfos;
import com.fjhx.victoriatourist.mapper.deliver.DeliverGoodsMapper;
import com.fjhx.victoriatourist.service.deliver.DeliverGoodsDetailsService;
import com.fjhx.victoriatourist.service.deliver.DeliverGoodsService;
import com.fjhx.victoriatourist.service.logistics.LogisticsDetailsService;
import com.fjhx.victoriatourist.service.logistics.LogisticsInfosService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 发货 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-13
 */
@DS(SourceConstant.VICTORIATOURIST)
@Service
public class DeliverGoodsServiceImpl extends ServiceImpl<DeliverGoodsMapper, DeliverGoods> implements DeliverGoodsService {
    @Autowired
    DeliverGoodsDetailsService deliverGoodsDetailsService;
    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    PurchaseDetailService purchaseDetailService;
    @Autowired
    LogisticsInfosService logisticsInfosService;
    @Autowired
    LogisticsDetailsService logisticsDetailsService;
    @Autowired
    PurchaseService purchaseService;

    @Override
    public Page<DeliverGoodsVo> getPage(DeliverGoodsSelectDto dto) {
        IWrapper<DeliverGoods> wrapper = getWrapper();
        wrapper.orderByDesc("dg", DeliverGoods::getId);
        Page<DeliverGoodsVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public List<DeliverGoodsDetailsVo> detail(Long id) {
        //根据发货id查发货明细
        IWrapper<DeliverGoods> wrapper = IWrapper.getWrapper();
        wrapper.eq("dgd.deliver_goods_id", id);
        wrapper.eq("dg.status", 0);
        List<DeliverGoodsDetailsVo> deliverGoodsDetailsVos = baseMapper.listDeliverGoodsDetails(wrapper);
        //获取采购明细列表
        //赋值产品信息
        List<Long> pdids = deliverGoodsDetailsVos.stream().map(DeliverGoodsDetails::getPurchaseDetailId).collect(Collectors.toList());
        if(ObjectUtil.isEmpty(pdids)){
            return deliverGoodsDetailsVos;
        }
        List<PurchaseDetail> purchaseDetailList = purchaseDetailService.listByIds(pdids);
        Map<Long, Long> goodidmap = purchaseDetailList.stream().collect(Collectors.toMap(PurchaseDetail::getId, PurchaseDetail::getBussinessId));
        List<Long> goodids = purchaseDetailList.stream().map(PurchaseDetail::getBussinessId).collect(Collectors.toList());
        List<ProductInfo> productInfos = productInfoService.listByIds(goodids);
        Map<Long, ProductInfo> productInfoMap = productInfos.stream().collect(Collectors.toMap(ProductInfo::getId, Function.identity()));
        //获取采购明细信息
        Map<Long, PurchaseDetail> purchaseDetailMap = purchaseDetailList.stream().collect(Collectors.toMap(PurchaseDetail::getId,Function.identity()));

        for (DeliverGoodsDetailsVo deliverGoodsDetailsVo : deliverGoodsDetailsVos) {
            //赋值产品信息
            Long goodid = goodidmap.get(deliverGoodsDetailsVo.getPurchaseDetailId());
            ProductInfo productInfo = productInfoMap.get(goodid);
            if (ObjectUtil.isNotEmpty(productInfo)) {
                deliverGoodsDetailsVo.setType(productInfo.getType());
                deliverGoodsDetailsVo.setCode(productInfo.getCode());
                deliverGoodsDetailsVo.setName(productInfo.getName());
                deliverGoodsDetailsVo.setSpec(productInfo.getSpec());
                deliverGoodsDetailsVo.setUnit(productInfo.getUnit());
            }
            //赋值采购数量
            PurchaseDetail purchaseDetail = purchaseDetailMap.get(deliverGoodsDetailsVo.getPurchaseDetailId());
            deliverGoodsDetailsVo.setCount(purchaseDetail.getCount());
            //赋值在途数量
            IWrapper<DeliverGoods> wrapper1 = IWrapper.getWrapper();
            wrapper1.eq("dgd",DeliverGoodsDetails::getPurchaseDetailId, deliverGoodsDetailsVo.getPurchaseDetailId());
            wrapper1.eq("dg.status",0);
            List<DeliverGoodsDetailsVo> list = baseMapper.listDeliverGoodsDetails(wrapper1);
            BigDecimal count = list.stream()
                    .map(DeliverGoodsDetails::getDeliverGoodsQuantity)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal subtract = count.subtract(deliverGoodsDetailsVo.getDeliverGoodsQuantity());
            deliverGoodsDetailsVo.setTransitQuantity(subtract);
        }
        return deliverGoodsDetailsVos;
    }

    @DSTransactional
    @Override
    public void add(DeliverGoodsDto deliverGoodsDto) {
        deliverGoodsDto.setStatus(0);//设置未到货
        this.save(deliverGoodsDto);

        //获取采购信息
        Purchase purchase = purchaseService.getById(deliverGoodsDto.getPurchaseId());
        //创建物流信息
        LogisticsInfos logisticsInfos = new LogisticsInfos();
        logisticsInfos.setBusinessId(deliverGoodsDto.getId());//设置业务id为发货id
        logisticsInfos.setBusinessCode(purchase.getCode());//设置采购单号为业务单号
        logisticsInfos.setBusinessType(1);//采购到货
        logisticsInfos.setStatus(0);
        logisticsInfos.setInStockStatus(10);
        logisticsInfos.setCode(deliverGoodsDto.getCode());
        logisticsInfos.setLogisticsCompanyCode(deliverGoodsDto.getLogisticsCompanyCode());
        logisticsInfos.setIsKd100(0);
        logisticsInfosService.save(logisticsInfos);

        List<DeliverGoodsDetails> deliverGoodsDetailsList = deliverGoodsDto.getDeliverGoodsDetailsList();
        List<LogisticsDetails> logisticsDetailsList = new ArrayList<>();

        for (DeliverGoodsDetails deliverGoodsDetails : deliverGoodsDetailsList) {
            deliverGoodsDetails.setDeliverGoodsId(deliverGoodsDto.getId());

            LogisticsDetails logisticsDetails = new LogisticsDetails();
            logisticsDetails.setBusinessDetailsId(deliverGoodsDetails.getId());//设置发货明细id为业务id
            logisticsDetails.setLogisticsInfoId(logisticsInfos.getId());
            logisticsDetails.setPurchaseId(purchase.getId());
            logisticsDetails.setPurchaseDetailsId(deliverGoodsDetails.getPurchaseDetailId());
            logisticsDetails.setShipmentQuantity(deliverGoodsDetails.getDeliverGoodsQuantity());
            logisticsDetailsList.add(logisticsDetails);
        }
        deliverGoodsDetailsService.saveBatch(deliverGoodsDetailsList);
        logisticsDetailsService.saveBatch(logisticsDetailsList);
    }

    @Override
    public void edit(DeliverGoodsDto deliverGoodsDto) {
        this.updateById(deliverGoodsDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<DeliverGoods> list(Long id) {
        //根据采购id查发货列表
        List<DeliverGoods> list = list(q -> q.eq(DeliverGoods::getPurchaseId, id).eq(DeliverGoods::getStatus, 0));
        return list;
    }

}
